#if defined __APPLE__
#  include "SamplesOSX.h"
#else
#  include "sample.h"
#endif

#include "sample_common.h"
#include "sprk.h"

#include <fstream>

using namespace HPS;

void Sample::Image_Io(WindowKey wk, Canvas canvas, View view, Model model, SegmentKey modelKey)
{

	/* Sample: Image_Io
	 * This sample will show you how to:
	 * - Load a JPG file through HPS::Image and check for Image IO exceptions
	 * - Obtain an ImageKit
	 * - Convert an image between formats
	 * - Output an image to a file
	 * - Insert text
	 * 
	 * Read more about the topics covered in this sample in our Programming Guide 
	 * - section 8.3:	Loading 2D Images */

	/* This next block of code shows you how to load an HSF file
	 * 1. Create an ImportOptionsKit and set the format of the image to be read in
	 * 2. Use the Image::File::Import method to load the desired image, with the options you specified
	 * 3. An ImageKit describing the inserted image is returned by Image::File::Import
	 * 4. Use a try-catch block to capture any Image IO exceptions */
	ImageKit jpgImageKit;
	unsigned int imageWidth, imageHeight;
	try
	{
		UTF8 file_in = INPUT_FOLDER + "/sprocket_image.jpg";
		Image::ImportOptionsKit importOptionsKit;
		importOptionsKit.SetFormat(Image::Format::Jpeg);
		jpgImageKit = Image::File::Import(static_cast<const char*>(file_in), importOptionsKit);
	}
	catch (IOException e)
	{
		UTF8 error_message;
		error_message = "Image Import Unsuccessful";
		modelKey.InsertText(Point(0, 0, 0), static_cast<const char*>(error_message));
		view.FitWorld();
		view.Update();
	}

	/* The next block of code shows how to convert between image formats and how to write an image to a file
	 * 1. Create a new ImageKit object and use its Convert method to change formats
	 * 2. The first argument of Convert is the source ImageKit
	 * 3. The second argument of Convert is the desired output format
	 * 4. Once the new ImageKit is obtained through Convert, obtain its data using the ImageKit::ShowData() method
	 * 5. Write the data out to a file in binary mode */
	if(!jpgImageKit.Empty())
	{
		bool exportSuccessful = true;
		ImageKit pngImageKit;
		ByteArray imageData;
		std::ofstream file;

		jpgImageKit.ShowSize(imageWidth, imageHeight);
		pngImageKit.SetFormat(Image::Format::Png).SetSize(imageWidth, imageHeight);
		pngImageKit.Convert(jpgImageKit, Image::Format::Png);
		pngImageKit.ShowData(imageData);
		
		UTF8 file_out = OUTPUT_FOLDER + "/image_io.png";

		file.open(static_cast<const char*>(file_out), std::ios::out | std::ios::binary);
		if(file.fail())
			exportSuccessful = false;

		else
		{
			size_t size = imageData.size();
			for(size_t i=0; i < size; i++)
			{
				file << imageData.at(i);
			}
		}
		if(file.fail())
			exportSuccessful = false;

		file.close();
		if(file.fail())
			exportSuccessful = false;

		if(!exportSuccessful)
		{
			UTF8 error_message;
			error_message = "Image Export Unsuccessful";
			modelKey.InsertText(Point(0, 0, 0), static_cast<const char*>(error_message));
			view.FitWorld();
			view.Update();
		}
		else
		{
			/* The next block of code shows you how to delete geometry from a segment and how to insert text
			 * 1. The Flush method can be used to delete different elements from a segment
			 * 2. The first argument of Flush specifies what to delete
			 * 3. The second argument specifies which part of the scene graph will be affected by this operation
			 * 4. To insert text, specify the text position and the text you want displayed*/
			UTF8 success_message = "Import and Export completed successfully\n" +
				(UTF8)"You can find the PNG image here:\n" + 
				file_out;

			view.GetAttachedModel().GetSegmentKey().Flush(Search::Type::Everything, Search::Space::SubsegmentsAndIncludes);
			view.GetAttachedModel().GetSegmentKey().InsertText(Point(0, 0, 0), static_cast<const char*>(success_message));
			view.FitWorld();
			view.Update();
		}
		
	}

	
}
